from celery import shared_task, group

from django.db import transaction
from django.utils import timezone

from allianceauth.eveonline.models import EveAllianceInfo
from allianceauth.services.hooks import get_extension_logger

from .models import AllianceContact, AllianceContactLabel, AllianceToken
from .provider import esi

logger = get_extension_logger(__name__)


@shared_task
def load_contact_name(contact_pk: int):
    contact = AllianceContact.objects.get(pk=contact_pk)
    contact.contact_name


@shared_task
def update_alliance_contacts(alliance_id: int):
    contact_to_load = []

    try:
        alliance = EveAllianceInfo.objects.get(alliance_id=alliance_id)
    except EveAllianceInfo.DoesNotExist:
        alliance = EveAllianceInfo.objects.create_alliance(alliance_id)

    try:
        alliance_token = AllianceToken.objects.with_valid_tokens().select_related('token').get(alliance=alliance)
    except AllianceToken.DoesNotExist:
        raise ValueError(f"No valid token found for alliance {alliance}")

    labels_data = (
        esi.client
        .Contacts
        .get_alliances_alliance_id_contacts_labels(
            alliance_id=alliance_id,
            token=alliance_token.token.valid_access_token()
        )
        .results()
    )

    labels = {label['label_id']: label['label_name'] for label in labels_data}

    contacts_data = (
        esi.client
        .Contacts
        .get_alliances_alliance_id_contacts(
            alliance_id=alliance_id,
            token=alliance_token.token.valid_access_token()
        )
        .results()
    )

    contact_ids = {
        contact['contact_id']: {
            'contact_type': contact['contact_type'],
            'label_ids': contact.get('label_ids', []),
            'standing': contact['standing']
        } for contact in contacts_data
    }

    with transaction.atomic():
        alliance_labels = AllianceContactLabel.objects.filter(
            alliance=alliance
        )

        alliance_labels.exclude(
            label_id__in=labels.keys()
        ).delete()

        for label_id, label_name in labels.items():
            label, _ = alliance_labels.update_or_create(
                alliance=alliance,
                label_id=label_id,
                defaults={'label_name': label_name}
            )

            labels[label_id] = label

        alliance_contacts = AllianceContact.objects.filter(alliance=alliance)

        alliance_contacts.exclude(
            contact_id__in=contact_ids.keys()
        ).update(standing=0.0)

        for contact_id, contact_data in contact_ids.items():
            contact, _ = alliance_contacts.update_or_create(
                contact_id=contact_id,
                defaults={
                    'contact_type': contact_data['contact_type'],
                    'standing': contact_data['standing']
                }
            )

            contact.labels.clear()
            if contact_data['label_ids'] is not None:
                contact.labels.set([labels[label_id] for label_id in contact_data['label_ids']])

            contact_to_load.append(contact.pk)

        alliance_token.last_update = timezone.now()
        alliance_token.save()

    group(load_contact_name.si(pk) for pk in contact_to_load).delay()


@shared_task
def update_all_alliances_contacts():
    for alliance_token in AllianceToken.objects.with_valid_tokens().select_related('alliance'):
        update_alliance_contacts.delay(alliance_token.alliance.alliance_id)



from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib import messages

from esi.decorators import token_required
from esi.models import Token
from allianceauth.eveonline.models import EveCharacter, EveAllianceInfo

from .models import AllianceContact, AllianceToken
from .tasks import update_alliance_contacts
from .forms import AllianceContactForm


@login_required
def index(request):
    return redirect('aa_contacts:contacts')


@login_required
@permission_required('aa_contacts.view_contacts')
def contacts(request):
    try:
        alliance = request.user.profile.main_character.alliance
    except EveAllianceInfo.DoesNotExist:
        alliance = None

    contacts = (
        AllianceContact.objects
        .filter(alliance=alliance)
        .prefetch_related('labels')
    )

    token = AllianceToken.objects.filter(alliance=alliance).first()

    context = {
        'contacts': contacts,
        'token': token,
        'alliance': alliance,
    }

    return render(request, 'aa_contacts/contacts.html', context=context)


@login_required
@permission_required('aa_contacts.view_contacts')
@token_required(scopes=['esi-alliances.read_contacts.v1'])
def add_token(request, token: Token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)

    if char.alliance_id is None:
        messages.error(request, 'You need to be in an alliance to add alliance contacts.')
        return redirect('aa_contacts:index')

    try:
        alliance = char.alliance
    except EveAllianceInfo.DoesNotExist:
        alliance = EveAllianceInfo.objects.create_alliance(char.alliance_id)

    if AllianceToken.objects.filter(alliance=alliance).exists():
        messages.error(request, 'Alliance contacts for your alliance are already being tracked.')
        return redirect('aa_contacts:index')

    AllianceToken.objects.create(alliance=alliance, token=token)
    update_alliance_contacts.delay(alliance.alliance_id)

    messages.success(request, 'Alliance contacts are now being tracked.')
    return redirect('aa_contacts:index')


@login_required
@permission_required('aa_contacts.view_contacts')
def update_alliance(request):
    try:
        alliance = request.user.profile.main_character.alliance
    except EveAllianceInfo.DoesNotExist:
        alliance = None

    if alliance is None:
        messages.error(request, 'You need to be in an alliance to update alliance contacts.')
        return redirect('aa_contacts:index')

    update_alliance_contacts.delay(alliance.alliance_id)

    messages.success(request, 'Alliance contacts are being updated.')
    return redirect('aa_contacts:index')


@login_required
@permission_required('aa_contacts.view_contacts')
def update_contact(request, contact_pk: int):
    contact = get_object_or_404(AllianceContact, pk=contact_pk)

    if request.method == 'POST':
        form = AllianceContactForm(request.POST, instance=contact)
        if form.is_valid():
            form.save()
            messages.success(request, f'{contact.contact_name} contact updated successfully')
            return redirect('aa_contacts:contacts')
    else:
        form = AllianceContactForm(instance=contact)

    context = {
        'form': form,
        'contact': contact,
    }

    return render(request, 'aa_contacts/edit_contact.html', context=context)



from django.urls import path

from . import views

app_name = 'aa_contacts'


urlpatterns = [
    path('', views.index, name='index'),
    path('contacts/', views.contacts, name='contacts'),
    path('add_token/', views.add_token, name='add_token'),
    path('update_alliance/', views.update_alliance, name='update_alliance'),
    path('edit_contact/<int:contact_pk>/', views.update_contact, name='update_contact'),
]
